// Copyright © Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.provider.aws.security;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

import java.nio.ByteBuffer;

import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedConstruction;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.core.aws.ssm.K8sLocalParameterProvider;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.powermock.reflect.Whitebox;

import com.amazonaws.SdkClientException;
import com.amazonaws.services.kms.AWSKMS;;

@RunWith(MockitoJUnitRunner.class)
public class KmsHelperTest {
	
    @InjectMocks
    private KmsHelper kmsHelper;
    
    @Mock
    private DpsHeaders dpsHeaders;
    
    @Mock
    private ByteBuffer ciphertext;
    
    private static MockedConstruction<K8sLocalParameterProvider> mockedConstruction;

    
    @BeforeClass
    public static void setup() {
        mockedConstruction = Mockito.mockConstruction(K8sLocalParameterProvider.class,
                                 (mock, context) -> {
                                     //implement initializer for mock. Set return value for object A mock methods
                                     when(mock.getParameterAsString("notification-sqs-url")).thenReturn(
                                         "test-sqs-url");
                                 });
    }

    @AfterClass
    public static void close(){
        mockedConstruction.close();
    }
    
    @Before
    public void initTest() {
        Whitebox.setInternalState(kmsHelper, "amazonRegion", "us-east-1");
        Whitebox.setInternalState(kmsHelper, "kmsEndpoint", "aws.kms.endpoint");
    }

    @Test(expected = SdkClientException.class)
    public void encrypt_EncryptsData() {
    	kmsHelper.init();
    	kmsHelper.encrypt("plain text");
    }
    
    @Test(expected = SdkClientException.class )
    public void decryptThorwsFormMockedKmsClient() {
    	kmsHelper.init();
    	kmsHelper.decrypt(ciphertext, "dataPartitionId");
    }
}
