/* Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.example;

import org.opengroup.osdu.core.common.cryptographic.SignatureService;
import org.opengroup.osdu.core.common.cryptographic.SignatureServiceException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.annotation.RequestScope;

import jakarta.servlet.http.HttpServletRequest;

import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.Map;

@Component("hmacVerification")
@RequestScope
public class HmacVerification {
    @Autowired
    private HttpServletRequest request;

    private final SignatureService service = new SignatureService();

    private static final Logger LOGGER = LoggerFactory.getLogger(HmacVerification.class);

    public boolean verifyHmac() {
        boolean foundError = false;
        LOGGER.info("Inside verifyHmac");
        LOGGER.error("Request: {}", request);
        try {
            Map<String, String> params = QueryStringUtils.parseQueryString(request.getQueryString());
            LOGGER.info(request.getQueryString());
            String tokenToVerify = params.get("hmac");
            if (tokenToVerify == null) throw new SignatureServiceException("Hmac verification not attached to request!");
            tokenToVerify = URLDecoder.decode(tokenToVerify, StandardCharsets.UTF_8);
            service.verifyHmacSignature(tokenToVerify, OSDUSecretString.getSecret());
        } catch (SignatureServiceException except) {
            LOGGER.error("Failed to verify request.", except);
            foundError = true;
        }
        return !foundError;
    }

    private static final String URL = "https://my.url/com";
    private static final String NONCE = "FEDCBA9876543210";
    public static void main(String[] args) throws SignatureServiceException {
        SignatureService service = new SignatureService();
        String expireTime = Long.toString(System.currentTimeMillis() + 3600000);
        String data = String.format("{\"expireMillisecond\": \"%s\",\"hashMechanism\": \"hmacSHA256\",\"endpointUrl\": \"%s\",\"nonce\": \"%s\"}", expireTime, URL, NONCE);
        String encodedData = Base64.getEncoder().encodeToString(data.getBytes(StandardCharsets.UTF_8));
        String signature = encodedData + "." + service.getSignedSignature(URL, OSDUSecretString.getSecret(), expireTime, NONCE);
        System.out.println(signature);
    }
}
